/*
*  Rubrica
*  file: import.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <gnome.h>

#include "rubrica.h"
#include "log.h"
#include "file.h"
#include "main.h"
#include "dialog.h"
#include "import.h"
#include "import_int.h"
#include "vcard.h"
#include "csv.h"
#include "view.h"
#include "card.h"

static gchar *gnomecard = ".gnome/GnomeCard.gcrd";
static gchar *kdecard   = ".kde/share/apps/kabc/std.vcf";

RubricaImportType importfrom;


void rubrica_import_init(GtkWidget *fentry, RubricaImportType import)
{  
  importfrom = import; 
}

void rubrica_import_set(RubricaImportType import)
{
  importfrom = (RubricaImportType) import;  
}

RubricaImportType rubrica_import_get(void)
{
  return (RubricaImportType) importfrom;
}


gboolean rubrica_import_data(GtkWidget *app)
{
  Rubrica *rubrica;
  GtkWidget *fentry;
  gboolean imported = TRUE;
  gchar *filename = NULL;
  gchar *msg, *buffer, *titolo = NULL;
  
  fentry  = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				       (rubrica_import_get_file_entry()));
  filename = g_strdup(gtk_entry_get_text(GTK_ENTRY(fentry)));
 
  switch (importfrom)
    {
    case IMPORT_GNOMECARD:
      /* 	 if no name is set in gnome_entry, then 
		 load default gnomecard addressbook 
      */
      if (strcmp(filename, "") == 0)
	{
	  /* load the default GnomeCard file: GnoneCard.gcrd*/
	  buffer = g_strdup(g_get_home_dir());	  
	  filename = g_strdup_printf("%s%s%s", buffer, G_DIR_SEPARATOR_S, 
				     gnomecard);

	  if (!g_file_exists(filename))
	    {
	      rubrica_dialog_error_loading_file(filename, RE_FILE_NOT_EXIST);

	      g_free(buffer);
	      g_free(filename);
	      return FALSE;
	    }

	  msg = g_strdup_printf("\nimporting gnomecard file: %s", filename);
	  rubrica_log_write_log(msg);
	  g_free(msg);

	  rubrica_import_open_vcard_file(filename);
	  titolo = g_strdup("GnomeCard.rub");

	  g_free(buffer);
	  g_free(filename);	  
	}
      else	/* import a gnomecard file */
	{
	  gboolean ret;

	  msg = g_strdup_printf("\nimporting gnomecard file: %s", filename); 
	  rubrica_log_write_log(msg);
	  g_free(msg);

	  ret = rubrica_import_open_vcard_file(filename);

	  g_free(filename);
	  return ret;
	}
      break;
      
    case IMPORT_KAB:
      buffer =  g_strdup(g_get_home_dir());      
      filename = g_strdup_printf("%s%s%s",buffer, G_DIR_SEPARATOR_S, kdecard);

      if (!g_file_exists(filename))
	{
	  rubrica_dialog_error_loading_file(filename, RE_FILE_NOT_EXIST);

	  g_free(buffer);
	  g_free(filename); 
	  return FALSE;
	}
      
      msg = g_strdup_printf("\nimporting Kabc file: %s", filename);      
      rubrica_log_write_log(msg);
      g_free(msg);

      rubrica_import_open_vcard_file(filename);
      titolo = g_strdup("std.rub");

      g_free(buffer);
      g_free(filename);      
      break;

    case IMPORT_RUBRICA:
      if (strcmp(filename, "") == 0)
	{
	  rubrica_dialog_error_loading_file(NULL, RE_SELECT_FILE);
	  
	  return FALSE;
	}
      else 	/* apri il file e leggilo */
	{
	  msg = g_strdup_printf("\nimporting rubrica's file: %s", filename);
	  rubrica_log_write_log(msg);
	  g_free(msg);

	  rubrica_import_open_old_rubrica_file(filename);

	  g_free(filename);
	}
      break;

    case IMPORT_CSV_MOZILLA:
    case IMPORT_CSV_NETSCAPE:
      if (strcmp(filename, "") == 0)
	{
	  rubrica_dialog_error_loading_file(NULL, RE_SELECT_FILE);

	  return FALSE;
	}
      else 
	{
	  msg = g_strdup_printf("\nimporting csv file: %s", filename);
	  rubrica_log_write_log(msg);
	  g_free(msg);

	  imported = rubrica_import_open_csv_file(filename);      
	  
	  if (!imported)
	    return FALSE;

	  titolo = g_strdup_printf("%s.rub", filename);	  

	  g_free(filename);
	}
      break;      

    case IMPORT_VCARD:
      if (strcmp(filename,"") == 0)
	{
	  rubrica_dialog_error_loading_file(NULL, RE_SELECT_FILE);

	  return FALSE;
	}
      else 
	{
	  msg = g_strdup_printf("\nimporting vcard file: %s", filename);
	  rubrica_log_write_log(msg);
	  g_free(msg);

	  rubrica_import_open_vcard_file(filename);     
	  titolo = g_strdup_printf("%s.rub", filename);	

	  g_free(filename);
	}
      break;      

    case IMPORT_PINE:
      rubrica_dialog_will_do();

      return FALSE;
      break;
      
    default:
      return FALSE;
      break;

    }

  rubrica = rubrica_get_current_rubrica();
  
  rubrica_set_path(rubrica, g_dirname(filename));
  rubrica_set_name(rubrica, g_basename(filename));
  rubrica_set_new(rubrica, FALSE);
  rubrica_set_empty(rubrica, FALSE);
  rubrica_set_modified(rubrica, TRUE);
  
  rubrica_card_set_tab_name(titolo);
  rubrica_view_set_items(rubrica_get_items(rubrica));
  
  if (titolo)
    g_free(titolo);
  
  return TRUE;
}


gboolean rubrica_import_open_vcard_file(gchar *file)
{
  Rubrica *rubrica;
  FILE *fp;

  if ((fp=fopen(file, "r")) == NULL)
    {
      rubrica_dialog_error_loading_file (file, RE_IMPORT_FILE);

      return FALSE;
    }

  rubrica_card_add_page_view(file);
  rubrica = rubrica_get_current_rubrica();

  rubrica_vcard_import_file(rubrica, fp);

  return TRUE;
}

gboolean rubrica_import_open_csv_file(gchar *file)
{
  FILE *fp;

  if ((fp=fopen(file, "r")) == NULL)
    {
      rubrica_dialog_error_loading_file (file, RE_IMPORT_FILE);

      return FALSE;
    }

  if (rubrica_csv_import_file(fp))
    return TRUE;

  return FALSE;
}

gint rubrica_import_open_old_rubrica_file(gchar *fname)
{
  RubricaErrorType err;
  gboolean loaded;
  
  loaded = rubrica_file_load_xmlfile(fname, &err);

  if (!loaded)
    {
      rubrica_dialog_error_loading_file(fname, err);      

      return 0;
    }
  
  return (gint) rubrica_view_get_indirizzi();    
}



