/*
*  Rubrica
*  file: vcard.c
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <ctype.h>
#include <time.h>

#include "rubrica.h"
#include "vcard.h"

#define ARRAY_SIZE 32

RubricaItem *r;

GList *web=NULL, *email=NULL, *tel=NULL;
GList *notelst;
TelNum *tt;
gboolean inline_binary_data = FALSE;

static RubricaVCardParamType parray[ARRAY_SIZE];
static RubricaVCardValueType varray[ARRAY_SIZE];

gint pindex = 0, vindex = 0;


RubricaVCardType tokens[] = { 
  {"N",   N},             {"FN",  FN},        {"TZ",  TZ},
  {"END", END},           {"ADR", ADR},       {"TEL", TEL},
  {"GEO", GEO},           {"ORG", ORG},       {"REV", REV},
  {"UID", UID},           {"URL", URL},       {"KEY", KEY},
  {"BDAY", BDAY},         {"ROLE", ROLE},     {"LOGO", LOGO}, 
  {"NOTE", NOTE},         {"BEGIN", BEGIN},   {"PHOTO", PHOTO},  
  {"LABEL", LABEL},       {"EMAIL", EMAIL},   {"TITLE", TITLE}, 
  {"AGENT", AGENT},       {"PROID", PROID},   {"SOUND", SOUND},  
  {"CLASS", CLASS},       {"MAILER", MAILER}, {"VERSION", VERSION}, 
  {"NICKNAME", NICKNAME}, {"CATEGORIES", CATEGORIES},

  /* kde extension.
  */
  {"X-KADDRESSBOOK-X-Profession",     KDE_EXT_Profession},
  {"X-KADDRESSBOOK-X-Department",     KDE_EXT_Department},
  {"X-KADDRESSBOOK-X-ManagersName",   KDE_EXT_ManagersName},
  {"X-KADDRESSBOOK-X-AssistantsName", KDE_EXT_AssistantsName},
  {"X-KADDRESSBOOK-X-Office",         KDE_EXT_Office},
  {"X-KADDRESSBOOK-X-SpousesName",    KDE_EXT_SpousesName},
  {"X-KADDRESSBOOK-X-Anniversary",    KDE_EXT_Anniversary},
  {"X-KADDRESSBOOK-X-CUSTOM",         KDE_EXT_CUSTOM},
  
  /* rubrica extension. 
     So rubrica can import a previously 
     exported rubrica's addressbook to vcard format
  */
  {"X-RUBRICA-X-Group",          RUBRICA_EXT_Group},
  {"X-RUBRICA-X-CompanyName",    RUBRICA_EXT_CompanyName},
  {"X-RUBRICA-X-Street",         RUBRICA_EXT_Street},
  {"X-RUBRICA-X-StreetNumber",   RUBRICA_EXT_StreetNumber},
  {"X-RUBRICA-X-ZipCode",        RUBRICA_EXT_ZipCode},
  {"X-RUBRICA-X-City",           RUBRICA_EXT_City},
  {"X-RUBRICA-X-Province",       RUBRICA_EXT_Province},
  {"X-RUBRICA-X-Country",        RUBRICA_EXT_Country},
  {"X-RUBRICA-X-Web",            RUBRICA_EXT_Web},
  {"X-RUBRICA-X-Email",          RUBRICA_EXT_Email},
  {"X-RUBRICA-X-Operator",       RUBRICA_EXT_Operator},
  {"X-RUBRICA-X-Fax",            RUBRICA_EXT_Fax},
  {"X-RUBRICA-X-Green",          RUBRICA_EXT_Green},
  {"X-RUBRICA-X-CustomerCare",   RUBRICA_EXT_CustomerCare},
  {"X-RUBRICA-X-Notes",          RUBRICA_EXT_Notes},
  {"X-RUBRICA-X-Department",     RUBRICA_EXT_Department},
  {"X-RUBRICA-X-SubDepartment",  RUBRICA_EXT_SubDepartment},
  {"X-RUBRICA-X-SecretaryName",  RUBRICA_EXT_SecretaryName},
  {"X-RUBRICA-X-SecretaryPhone", RUBRICA_EXT_SecretaryPhone},
  {"X-RUBRICA-X-SpouseName",     RUBRICA_EXT_SpouseName},
  {"X-RUBRICA-X-Child",          RUBRICA_EXT_Child},
  {"X-RUBRICA-X-Hobbies",        RUBRICA_EXT_Hobbies},
  {NULL,  BAD_TOKEN}
};
	
	
RubricaVCardType paramtokens[] = {
  {"type",     TYPE},
  {"value",    VALUE},   
  {"encoding", ENCODING},
  {NULL,       BAD_PARAM}
};		


RubricaVCardType valuetokens[] = {
  {"home",     HOME},     {"work",       WORK},       {"cell",    CELL},     
  {"fax",      FAX},      {"postal",     POSTAL},     {"parcel",  PARCEL},  
  {"dom",      DOM},      {"intl",       INTL},       {"pref",    PREF},    
  {"voice",    VOICE},    {"msg",        MSG},        {"pager",   PAGER},   
  {"bbs",      BBS},      {"modem",      MODEM},      {"car",     CAR},     
  {"isdn",     ISDN},     {"video",      VIDEO},      {"aol",     AOL},     
  {"internet", INTERNET}, {"applelink",  APPLELINK},  {"attmail", ATTMAIL},
  {"cis",      CIS},      {"eworld",     EWORLD},     {"ibmmail", IBMMail},  
  {"mcimail",  MCIMAIL},  {"powershare", POWERSHARE}, {"prodigy", PRODIGY}, 
  {"tlx",      TLX},      {"x400",       X400},       {"vcard",   VCARD},
  {"pgp",      PGP},      {"x509",       X509},       {"b",       BINARY},   
  {"binary",   BINARY},   {"text",       TEXT},       {"uri",     URI},
  {"basic",    BASIC},    {"jpeg",       JPEG},       {"public",  PUBLIC},
  {"private",  PRIVATE},  {"confidential", CONFIDENTIAL}, 
  {"quoted-printable", QUOTED_PRINTABLE},  
  {NULL,       BAD_VALUE}
};


void rubrica_vcard_import_file(Rubrica *rubrica, FILE *fin)
{
  RubricaVCardParseState state;
  RubricaVCardTokenType token;
  RubricaVCardParamType param;
  RubricaVCardValueType value;

  parray[ARRAY_SIZE] = BAD_PARAM;
  varray[ARRAY_SIZE] = BAD_VALUE;

  gchar buffer_token[32], buffer_param[32], buffer_val[32]; 
  gchar data[1024];
//  gchar *binary_data;
//  glong bytes;
//  gint count = 0;
  gint ch;
  gint i = 0, j = 0, k = 0, l = 0, p = 0;  
  gboolean new_card = TRUE;

  state = BUILD_TOKEN_STATE;
  
  while ((ch = fgetc(fin)))
    {
      if (ch == EOF)
	break;
     
      if (new_card)
	{
	  for (p = 0; p < ARRAY_SIZE; p++)
	    {
	      parray[p] = BAD_PARAM;
	      varray[p] = BAD_VALUE;
	    }
	  pindex = 0;
	  vindex = 0;

	  new_card = FALSE;
	}

      switch (state)
	{
	case BUILD_TOKEN_STATE:
	  if (g_unichar_isalnum(ch) || ch == '-')
	    buffer_token[i++] = (gchar) ch;
	  
	  /* look the next character */
	  ch = fgetc(fin);
	  
	  if (ch == ';')
	    {	    
	      state = BUILD_PARAM_STATE;

	      buffer_token[i] = '\0';
	      i = 0;
	      
	      /* token has been read. Decode it */
	      token = rubrica_vcard_decode_token(buffer_token);
	    }
	  else if (ch == ':')
	    {
	      state = BUILD_DATA_STATE;

	      buffer_token[i] = '\0';
	      i = 0;
      
	      /* decode token */
	      token = rubrica_vcard_decode_token(buffer_token);
	    }

	  else ungetc(ch, fin);
	  break;
	  
	case BUILD_PARAM_STATE:
	  if (g_unichar_isalnum(ch) || ch == '-')
	    buffer_param[j++] = (gchar) ch;
	  
	  /* test next char */
	  ch = fgetc(fin);
	  
	  if (ch == '=')
	    {
	      state = BUILD_VALUE_STATE;
	      buffer_param[j] = '\0';
	      j=0;
	      
	      /* decodificare param */
	      parray[pindex++] = rubrica_vcard_decode_param(buffer_param);
	    }
	  else if (ch == ',' || ch == ';')
	    {
	      state = BUILD_VALUE_STATE;
	      buffer_param[j] = '\0';
	      j=0;

	      /* decodificare val */
	      value = rubrica_vcard_decode_value(buffer_param);
	      if (value == BINARY || value == JPEG)
		inline_binary_data = TRUE;

	      varray[vindex++] = value;
	    }
	  else if (ch == ':')
	    {
	      state = BUILD_DATA_STATE;
	      buffer_param[j] = '\0';
	      j=0;

	      /* decodificare val */
	      value = rubrica_vcard_decode_value(buffer_param);
	      if (value == BINARY || value == JPEG)
		inline_binary_data = TRUE;

	      varray[vindex++] = value;
	    }
	  else ungetc(ch, fin);
	  break;
	  
	case BUILD_VALUE_STATE:
	  if (g_unichar_isalnum(ch) || ch == '-')
	    buffer_val[k++] = (gchar) ch;
	  
	  /* test next char */
	  ch = fgetc(fin);
	  
	  if (ch == ',')
	    {
	      state = BUILD_VALUE_STATE;
	      buffer_val[k] = '\0';
	      k = 0;
	      
	      /* decodificare val */
	      value = rubrica_vcard_decode_value(buffer_val); 
	      if (value == BINARY || value == JPEG)
		inline_binary_data = TRUE;

	      varray[vindex++] = value;
	    }
	  else if (ch == ';')
	    {
	      state = BUILD_PARAM_STATE;
	      buffer_val[k] = '\0';
	      k = 0;
	      
	      /* decodificare val */
	      value = rubrica_vcard_decode_value(buffer_val); 		      
	      if (value == BINARY || value == JPEG)
		inline_binary_data = TRUE;

	      varray[vindex++] = value;
	    }
	  else if (ch == ':')
	    {
	      state = BUILD_DATA_STATE;
	      buffer_val[k] = '\0';
	      k = 0;
	      
	      /* decodificare val */
	      value = rubrica_vcard_decode_value(buffer_val); 		      
	      if (value == BINARY || value == JPEG)
		inline_binary_data = TRUE;

	      varray[vindex++] = value;
	    }
	  else ungetc(ch, fin);
	  break;
	  
	case BUILD_DATA_STATE:	  
	  if (inline_binary_data)
	    {
	      g_print("\nrubrica reads binary data from file (type: %s).\n"
		      "This rubrica's version don't use these data.\n",
		      (token == PHOTO ? "PHOTO" : "SOUND"));

	      while ((ch = fgetc(fin)) != '\n');

/*	      bytes = ftell(fin);
	      while ((ch = fgetc(fin)) != '\n')
		count++;

	      binary_data = g_malloc (count*sizeof(char));
	      fseek(fin, bytes, SEEK_SET);

	      count = 0;
	      while ((ch = fgetc(fin)) != '\n')
		binary_data[count++] = ch;

	      rubrica_vcard_build_card(rubrica, token, param_array, 
				       value_array, binary_data);

	      count = 0;
*/	      new_card = TRUE; 
	      inline_binary_data = FALSE; 
	      for (p = 0; p <= ARRAY_SIZE; p++)
		{
		  parray[p] = BAD_PARAM;
		  varray[p] = BAD_VALUE;
		  vindex = pindex = 0;
		}
	      state = BUILD_TOKEN_STATE; 

	      break;
	    }

	  data[l++] = (gchar) ch;
	  
	  /* test next char */
	  ch = fgetc(fin);  
	  if (ch == '\0' || ch == '\n' || ch == '\r')
	    {
	      data[l] = '\0';
	      l = 0;

	      /* salva i valori decodificati e torna 
		 allo stato 1 per una nuova riga*/
	      rubrica_vcard_build_card(rubrica, token, parray, varray, data);

	      new_card = TRUE;
	      for (p = 0; p <= ARRAY_SIZE; p++)
		{
		  parray[p] = BAD_PARAM;
		  varray[p] = BAD_VALUE;
		  vindex = pindex = 0;
		}
	      state = BUILD_TOKEN_STATE;
	    }	  
	  else ungetc(ch, fin);
	
	  break;
	  
	default:
	  break;	  
	}
    }  
}

RubricaVCardTokenType rubrica_vcard_decode_token(gchar *string)
{
  gint i=0;
  
  if (!string)
    return (RubricaVCardTokenType) BAD_TOKEN;  
    
  while (tokens[i].str)
    {
      if (string && ((strcasecmp(string, tokens[i].str)) == 0))
	return (RubricaVCardTokenType) tokens[i].tok;
      else
	i++;
    }

  return (RubricaVCardTokenType) BAD_TOKEN;
}


RubricaVCardParamType rubrica_vcard_decode_param(gchar *string)
{
  gint i=0;
  
  if (!string)
    return (RubricaVCardParamType) BAD_PARAM;  
    
  while (paramtokens[i].str)
    {
      if (string && ((strcasecmp(string, paramtokens[i].str)) == 0))
	return (RubricaVCardParamType) paramtokens[i].tok;
      else
	i++;
    }

  return (RubricaVCardParamType) BAD_PARAM;
}


RubricaVCardValueType rubrica_vcard_decode_value(gchar *string)
{
  gint i=0;
  
  if (!string)
    return (RubricaVCardValueType) BAD_VALUE;  
    
  while (valuetokens[i].str)
    {
      if (string && ((strcasecmp(string, valuetokens[i].str)) == 0))
	return (RubricaVCardValueType) valuetokens[i].tok;
      else
	i++;
    }

  return (RubricaVCardValueType) BAD_VALUE;
}


void rubrica_vcard_decode_data_buffer(char *buf, gchar *dec[7])
{
  gint i, j, k;
  gchar *tmp;

  /* inizializzazione buffer decodificato */
  for(i=0; i<8;i++)
    dec[i]=NULL;

  tmp = g_malloc (strlen(buf));
  i = j = k = 0;
  while (buf[j] != '\0')
    {
      if(buf[j] != ';')
	{
	  tmp[k++] = buf[j++];
	}	 
      else
	{
	  tmp[k]='\0';
	  dec[i] = g_strdup(tmp);
	  i++;
	  j++;
	  k=0;
	}		
    }
  tmp[k]='\0';
  dec[i] = g_strdup(tmp); 
}


time_t rubrica_vcard_decode_date(gchar *date, gboolean has_hour)
{
  /* date: 2001-10-21T15:53:29
           1973-03-18
  */

  struct tm tm;
  time_t t;
  gchar *str;
  
  str = g_strdup(date);
  strtok(date, "-");
  tm.tm_year = atoi(str);
  strtok(NULL, "-");
  tm.tm_mon = atoi(str);
  strtok(NULL, "T");
  tm.tm_mday = atoi(str);

  if (has_hour)
    {
      strtok(NULL, ":");
      tm.tm_hour = atoi(str);
      strtok(NULL, ":");
      tm.tm_min = atoi(str);
      strtok(NULL, "");
      tm.tm_sec = atoi(str);
    }
  
  t = mktime(&tm);

  return t;
}


void rubrica_vcard_build_card(Rubrica *rubrica, RubricaVCardTokenType token,
			      RubricaVCardParamType *parray, 
			      RubricaVCardValueType *varray, gchar *data)
{
//  RubricaVCardParamType param;
  RubricaVCardValueType value;
  gchar *decoded[8];
  gchar *tmp = NULL/*, *tmp1, *tmp2*/;
  gboolean have_adr = FALSE;
  gint index = 0;
  time_t t;
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(data  != NULL);

  switch(token)
    {
    case BEGIN:
      r = rubrica_item_new();
      if (!r) return;
 
      web = email = tel = NULL; 
      break;
      
    case END:
      /* link web, email and tel lists
       */
      r->net.web         = web;
      r->net.email       = email;
      r->phone.telephone = tel;

      /* inserire l'elemento nella lista rubrica 
       */
      rubrica_append_item(rubrica, r);
      break;

    case FN:
      /* card name */ 
      r->card = g_strdup(data);      
      break;
      
    case N:
      /* ord:: surname, second name, prefix, suffix 
	       cognome, nome, secondo nome, prefisso, suffisso 
      */
      rubrica_vcard_decode_data_buffer(data, decoded);
 
      r->personal.last   = (decoded[0] ? g_strdup(decoded[0]) : g_strdup(""));
      r->personal.first  = (decoded[1] ? g_strdup(decoded[1]) : g_strdup(""));
      r->personal.middle = (decoded[2] ? g_strdup(decoded[2]) : g_strdup(""));
      r->personal.prefix = (decoded[3] ? g_strdup(decoded[3]) : g_strdup(""));

      /* decoded[4] <= suffix, don't used by rubrica */
      break;

    case TZ:
      /* Time Zone 
      tmp1 = r->notes.notes;
      tmp2 = (tmp1 ? g_strconcat(tmp1, "\nTime Zone: ", data, NULL) 
	      : g_strconcat("\nTime Zone: ", data, NULL));

      if (tmp1)
	g_free(tmp1);      
      r->notes.notes = tmp2;
      */
      break;

    case REV:
      /* date, fuso orario, time
	 data, fuso orario, ora 
      */
      t = rubrica_vcard_decode_date(data, TRUE);
      break;

    case ADR:
      /* mail address + mail type 
	 indirizzo di posta + tipo 
      */
      if (have_adr)  /* now rubrica store only the first address that meet */
	break;
      else
	have_adr = TRUE;

      rubrica_vcard_decode_data_buffer(data, decoded);

      r->address.street   = (decoded[2] ? g_strdup(decoded[2]) : g_strdup(""));
      r->address.number   = g_strdup("");
      r->address.zip      = (decoded[5] ? g_strdup(decoded[5]) : g_strdup(""));
      r->address.city     = (decoded[3] ? g_strdup(decoded[3]) : g_strdup(""));
      r->address.province = (decoded[4] ? g_strdup(decoded[4]) : g_strdup(""));
      r->address.country  = (decoded[6] ? g_strdup(decoded[6]) : g_strdup(""));
      r->address.state    = g_strdup("");

      /* 
	 decoded[0] <= post office, don't used by rubrica 
	 decoded[1] <= extended,    don't used by rubrica 
      */
      if (decoded[0])
	{
	  // 	  aggiungere alle note personali
	  /*
	  tmp = g_strdup_printf(_("Post Office: %s\n"),decoded[0]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);      
	  */
	}
      if (decoded[1])
	{
	  /*	    aggiungere alle note personali
	   */
	  /*
	  tmp = g_strdup_printf(_("Extended: %s\n"),decoded[1]);
	  notelen += strlen(tmp);
	  notelst = g_list_append(notelst, tmp);      
	  */
	}
      break;

    case TEL:
      /* telefono + tipo */
      tt = (TelNum*) g_malloc(sizeof(TelNum));
      tt->number = g_strdup(data);

      if (index < ARRAY_SIZE && 
	  (value = (RubricaVCardValueType) varray[index++]) != BAD_VALUE)
	{
	  g_print("\ntelefono: %d tipo: %d",  
		  tt->number,
		  (RubricaVCardValueType) value);
	}
      else
	{
	  g_print("\nbreak");
	  break;
	}
      
      switch (value)
	{	  
	case HOME:
	  tt->type = "home";
	  break;
	case WORK:
	  tt->type = "work";
	  break;
	case FAX:
	  tt->type = "fax";
	  break;
	case CELL:
	  tt->type = "cellphone";
	  break;
	  
	  /* other telephone type */
	case PREF:
	case VOICE:
	case MSG:
	case PAGER:
	case BBS:
	case MODEM:
	case CAR:
	case ISDN:
	case VIDEO:
	default:
	  tt->type = "other";
	  break;
	} 

      tel = g_list_append(tel, tt);	  
      break;
      
    case EMAIL:      
      tmp = g_strdup(data);
      email = g_list_append(email, tmp);	  
      break;

    case URL:      
      tmp = g_strdup(data);
      web = g_list_append(web, tmp);      
      break;

    case GEO:
      /* Geo Location (latitude, longitude) */
      /* load: long then latit 
         es. lati -0,3 long 5,4 ==> 5,4,-0,3 
      tmp1 = r->notes.notes;
      tmp2 = (tmp1 ? g_strconcat(tmp1, "\nGeo: ", data, NULL) 
	      : g_strconcat("\nGeo: ", data, NULL));
 
      if (tmp1)
	g_free(tmp1);      
      r->notes.notes = tmp2;
      */
      break;

    case BDAY:
      r->personal.birthdate = rubrica_vcard_decode_date(data, FALSE);
      break;
      
    case TITLE:
    case KDE_EXT_Profession:
      /* titolo nell'organizzazione (professione) */
      r->work.assigment = g_strdup(data);
      break;

    case ORG:
      /* nome dell'organizzazione */
      r->work.organization = g_strdup(data);
      break;

    case CATEGORIES:
      break;

    case NOTE: 
      /*
      tmp1 = r->notes.notes;
      tmp2 = (tmp1 ? g_strconcat(tmp1, "\nNotes: ", data, NULL) 
	      : g_strconcat("\nNotes: ", data, NULL));

      if (tmp1)
	g_free(tmp1);      
      r->notes.notes = tmp2;
      */
      break;
      
    case KEY: /* public key */
      tmp = g_strdup(data);
      r->notes.pubkey = tmp;
      break;

    case ROLE:
      tmp = g_strdup(data);
      r->work.assigment = tmp;
      break;

    case UID:
      break;

    case LOGO:
      break;

    case PHOTO:
      break;

    case LABEL:
      break;

    case AGENT:
      break;

    case PROID:
      break;

    case SOUND:
      break;

    case CLASS:
      break;

    case MAILER:
      break;

    case VERSION:
      break;

    case NICKNAME:
      break;
      
    case RUBRICA_EXT_Group:
      break;
  
    case RUBRICA_EXT_CompanyName:
      tmp = g_strdup(data);
      r->company.name = tmp;
      break;

    case RUBRICA_EXT_Street:
      tmp = g_strdup(data);
      r->company.street = tmp;
      break;
      
    case RUBRICA_EXT_StreetNumber:
      tmp = g_strdup(data);
      r->company.number = tmp;
      break;
      
    case RUBRICA_EXT_ZipCode:
      tmp = g_strdup(data);
      r->company.zip = tmp;
      break;
      
    case RUBRICA_EXT_City:
      tmp = g_strdup(data);
      r->company.city = tmp;
      break;
      
    case RUBRICA_EXT_Province:
      tmp = g_strdup(data);
      r->company.province = tmp;
      break;
      
    case RUBRICA_EXT_Country:
      tmp = g_strdup(data);
      r->company.country = tmp;
      break;
      
    case RUBRICA_EXT_Web:
      tmp = g_strdup(data);
      r->company.web = tmp;
      break;
      
    case RUBRICA_EXT_Email:
      tmp = g_strdup(data);
      r->company.email = tmp;
      break;
      
    case RUBRICA_EXT_Operator:
      tmp = g_strdup(data);
      r->company.operator = tmp;
      break;

    case RUBRICA_EXT_Fax:
      tmp = g_strdup(data);
      r->company.fax = tmp;
      break;
      
    case RUBRICA_EXT_Green:
      tmp = g_strdup(data);
      r->company.green = tmp;
      break;
      
    case RUBRICA_EXT_CustomerCare:
      tmp = g_strdup(data);
      r->company.customer_care = tmp;
      break;

    case RUBRICA_EXT_Notes:
      tmp = g_strdup(data);
      r->company.notes = tmp;
      break;

    case RUBRICA_EXT_Department:
    case KDE_EXT_Department:
      tmp = g_strdup(data);
      r->work.department = tmp;      
      break;
      
    case RUBRICA_EXT_SubDepartment:
      tmp = g_strdup(data);
      r->work.subdep = tmp;
      break;

    case RUBRICA_EXT_SecretaryName:
    case KDE_EXT_AssistantsName:
      tmp = g_strdup(data);
      r->work.secretary = tmp;
      break;
      
    case RUBRICA_EXT_SecretaryPhone:
      tmp = g_strdup(data);
      r->work.telephone = tmp;
      break;
      
    case RUBRICA_EXT_SpouseName:
    case KDE_EXT_SpousesName:
      tmp = g_strdup(data);
      
      r->notes.is_married  = TRUE;
      r->notes.spouse = tmp;      
      break;
      
    case RUBRICA_EXT_Child:
      tmp = g_strdup(data);
      
      r->notes.has_children = TRUE;
      r->notes.children = tmp;
      break;
      
    case RUBRICA_EXT_Hobbies:
      tmp = g_strdup(data);
      r->notes.hobbies = tmp;     
      break;

    case KDE_EXT_ManagersName:
      break;

    case KDE_EXT_Office:
      break;



    case  KDE_EXT_Anniversary:
      /*
      r->notes.birthknow   = FALSE;
      r->notes.anniverknow = TRUE;
      r->notes.anniverdate = rubrica_vcard_decode_complex_date(data, FALSE);  
      */
      break;

    case KDE_EXT_CUSTOM:  
      break;
        
    case BAD_TOKEN:      
    default:
      break;
    }
}




 
time_t rubrica_vcard_decode_complex_date(gchar *date)
{
  /* date format: week_day day month year
  */

  return 0;
}

