/*
*  Rubrica
*  file: vcard.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _VCARD_H
#define _VCARD_H

#include "rubrica.h"

typedef enum {
  N = 1, 
  FN, 
  TZ, 
  END, 
  ADR, 
  TEL, 
  GEO, 
  ORG, 
  REV, 
  UID, 
  URL, 
  KEY, 
  BDAY, 
  ROLE, 
  LOGO, 
  NOTE, 
  BEGIN,
  PHOTO, 
  LABEL,
  EMAIL, 
  TITLE, 
  AGENT, 
  PROID, 
  SOUND, 
  CLASS,
  MAILER, 
  VERSION,
  NICKNAME, 
  CATEGORIES,              // last vcard token
  KDE_EXT_Profession,      // KDE extensions
  KDE_EXT_Department,
  KDE_EXT_ManagersName,
  KDE_EXT_AssistantsName,
  KDE_EXT_Office,
  KDE_EXT_SpousesName,
  KDE_EXT_Anniversary,
  KDE_EXT_CUSTOM,
  RUBRICA_EXT_Group,       // Rubrica extensions
  RUBRICA_EXT_CompanyName,
  RUBRICA_EXT_Street,
  RUBRICA_EXT_StreetNumber,
  RUBRICA_EXT_ZipCode,
  RUBRICA_EXT_City,
  RUBRICA_EXT_Province,
  RUBRICA_EXT_Country,
  RUBRICA_EXT_Web,
  RUBRICA_EXT_Email,
  RUBRICA_EXT_Operator,
  RUBRICA_EXT_Fax,
  RUBRICA_EXT_Green,
  RUBRICA_EXT_CustomerCare,
  RUBRICA_EXT_Notes,
  RUBRICA_EXT_Department,
  RUBRICA_EXT_SubDepartment,
  RUBRICA_EXT_SecretaryName,
  RUBRICA_EXT_SecretaryPhone,
  RUBRICA_EXT_SpouseName,
  RUBRICA_EXT_Child,
  RUBRICA_EXT_Hobbies,

  BAD_TOKEN  // this isn't a VCard token
} RubricaVCardTokenType;


typedef enum {
  TYPE = 1, 
  VALUE, 
  ENCODING,
  BAD_PARAM   // this isn't a VCard token
} RubricaVCardParamType;


typedef enum {
  VCARD = 1,
  BINARY,
  BASIC, 
  PUBLIC, 
  PRIVATE, 
  CONFIDENTIAL,
  HOME,  
  WORK, 
  POSTAL, 
  PARCEL, 
  DOM, 
  INTL, 
  PREF,
  VOICE, 
  FAX, 
  MSG, 
  CELL, 
  PAGER, 
  BBS, 
  MODEM, 
  CAR, 
  ISDN, 
  VIDEO,  
  AOL, 
  INTERNET,
  APPLELINK, 
  ATTMAIL, 
  CIS, 
  EWORLD, 
  IBMMail, 
  MCIMAIL, 
  POWERSHARE, 
  PRODIGY, 
  TLX, 
  X400, 
  PGP, 
  X509, 
  TEXT,
  URI,
  JPEG,
  QUOTED_PRINTABLE,
  BAD_VALUE   // this isn't a vcard token
} RubricaVCardValueType;


typedef enum {
  BUILD_TOKEN_STATE,
  BUILD_PARAM_STATE,
  BUILD_VALUE_STATE,
  BUILD_DATA_STATE
} RubricaVCardParseState;



typedef struct _RubricaVCardType 
{
  gchar *str;        /* token */
  gint tok;          /* vcard decoded token */
} RubricaVCardType;



void rubrica_vcard_import_file(Rubrica *rubrica, FILE *fin);

RubricaVCardTokenType rubrica_vcard_decode_token(gchar *string);
RubricaVCardParamType rubrica_vcard_decode_param(gchar *string);
RubricaVCardValueType rubrica_vcard_decode_value(gchar *string);
void rubrica_vcard_decode_data_buffer(char *buf, gchar *dec[7]);
time_t rubrica_vcard_decode_date(gchar *date, gboolean has_hour);

void rubrica_vcard_build_card(Rubrica *rubrica, 
			      RubricaVCardTokenType token,
			      RubricaVCardParamType *parray, 
			      RubricaVCardValueType *varray,
			      gchar *data);

#endif
